/*
 * Copyright (C) 2000-2007 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */

/**
 * \file include/arch/btnode3/btn-hardware.h
 *
 * 26.05.2004 Martin Hinz <btnode@hinz.ch>
 * 23.05.2006 Jan Beutel <j.beutel@ieee.org>
 * 
 * @brief Platform dependent port connection for AVR ATmega128(L) and
 * BTnode Rev 3.10 and up.
 *
 * If you have to distinguish between BTnode rev2 and BTnode rev3 in
 * your program code use following if on def:\n
\verbatim
 #if defined(__BTNODE3__)
     program code for BTnode rev3
 #else
     program code for everything else (BTnode rev2, STK500, linux, ...)
 #endif 
\endverbatim
 */

// Internal Memory
#define NUTMEM_SIZE  4096
#define NUTMEM_START 0x100

// External Memory
#define NUTXMEM_START 0x1100
#define NUTXMEM_SIZE  0xEF00

// CPU Clock Frequency
#define CPU_FREQ 7372800

// CPU Type
#define MCU_ATMEGA128

// VCC in mV
#define VCC_mV 3300

/* 
 * we have a configuration latch on BTnode rev3.20 to control 
 * - LEDs
 * - BT power and reset
 * - CChipcon power
 * - IO power
 * 
 * but not on the BTnode rev3.10 where these pins are directly connected
 */
#define HAVE_CONFIG_LATCH

// initial value for the latch
#define CONFIG_LATCH_INIT_VALUE  0x00

// For testing.. there's no real latch and PORTC is just bridged
//#define CONFIG_LATCH_PATCHED

// set LATCH_SELECT pin B5
#define CONFIG_LATCH_SELECT_PORT PORTB
#define CONFIG_LATCH_SELECT_DDR DDRB
#define CONFIG_LATCH_SELECT_PIN 5

// VCC IO
#define IO_POWER_ON_PIN 4

//Ram port definitions
#define RAM_CTR_A_PORT PORTB
#define RAM_CTR_A_PIN 7
#define RAM_CTR_A_DDR DDRB
#define RAM_CTR_B_PORT PORTB
#define RAM_CTR_B_PIN 6
#define RAM_CTR_B_DDR DDRB


//Bluetooth ZV4002
#define BT_RADIO_TYPE ZEEVO
// #define BT_RADIO_ON_PORT PORTF   // not available on zv4002
// #define BT_RADIO_ON_PIN 0        // not available on zv4002
// #define BT_RADIO_ON_DDR DDRF     // not available on zv4002
// #define BT_POWER_ON_PORT PORTC   // on config latch
#define BT_POWER_ON_PIN 6
// #define BT_POWER_ON_DDR DDRF     // on config latch
// #define BT_RESET_PORT PORTC      // on config latch
#define BT_RESET_PIN 7
// #define BT_RESET_DDR DDRF        // on config latch

// Initial speed for the uart
#define BT_UART_INITIAL_SPEED 115200
// Parity for the uart (0-None, 1-Odd, 2-Even)
#define BT_UART_INITIAL_PARITY 0
#define BT_UART_INITIAL_DATABITS 8
#define BT_UART_INITIAL_STOPBITS 1
// Flowcontrol to use (UART_HS_RTSCTS or 0 for none)
#define BT_UART_INITIAL_FLOWCONTROL UART_HS_RTSCTS


/*
 * Chipcon CC1000 
 * \note On mica2, D4 is used for CC1000_PALE
 * \note On mica2, ADC #0 - Pin F0 is used for ADC reading 
 *
 */
#define CC1000_CONFIG_PORT      PORTD
#define CC1000_CONFIG_PORT_REG  DDRD
#define CC1000_PDATA            7 
#define CC1000_PCLK             6
#define CC1000_POWER_ON_PIN     5

#ifdef __MICA2__
#define CC1000_PALE             4
#define CC1000_RSSI_ADC         0
#else
#define CC1000_PALE             5
#define CC1000_RSSI_ADC         2
#endif

// Other ADCs
#define BAT_SENSE_ADC           3
#define BAT_DIVIDER             2


//I2C
// master trigger
#define I2C_MASTER_TRIGGER_PORT PORTD
#define I2C_MASTER_TRIGGER_PIN 6
#define I2C_MASTER_TRIGGER_DDR DDRD

//LED's
// we have LEDs on the CONFIG_LATCH PORTC
#define LED_PORT PORTC
#define LED_PORT_DDR DDRC
#define LED0    0
#define LED1    1
#define LED2    2
#define LED3    3

//define bt uart and app uart
#define APP_UART devUsartAvr0
#define APP_UART_NR 0
#define BT_UART devUsartAvr1
#define BT_UART_NR 1

//Uart0
// RxD: PE0
// TxD: PE1
// CTS: PE4 (external interrupt 4)
// RTS: PE2
#define UART0_CTS_IRQ INT4
#define UART0_RTS_BIT 2
#define UART0_RTS_AVRPORT AVRPORTE
//for compatibility reasons
#define UART0_CTS_DDR DDRE
#define UART0_CTS_BIT 4
#define UART0_RTS_PORT PORTE
#define UART0_RTS_DDR DDRE


//Uart1
// RxD: PD2
// TxD: PD3
// CTS: PE5 (external interrupt 5)
// RTS: PD4
#define UART1_CTS_IRQ INT5
#define UART1_RTS_BIT 4
#define UART1_RTS_AVRPORT AVRPORTD

// BT-UART latency optimization:
// - hardcode uart irq handler to usartavr
#define USE_USART1
// - read all bytes of uart rx buffer
#define UART1_READMULTIBYTE
// - strip code to handle software flowcontrol
#define UART1_NO_SW_FLOWCONTROL


//Software UART
// RxD default on: UART0_CTS (PE4)
#ifndef SUART_RXD_PORT
#define SUART_RXD_PORT PORTE
#define SUART_RXD_PIN PINE
#define SUART_RXD 4
#define SUART_RXD_INTERRUPT INT4
#define sig_SUART_RECV sig_INTERRUPT4
#endif
// TxD default on: UART0_RTS (PE2)
#ifndef SUART_TXD_PORT
#define SUART_TXD_PORT PORTE
#define SUART_TXD_PORT_DDR DDRE
#define SUART_TXD 2
#endif

// TecO Sensor Board
//#define TECO_POWER_ON_PIN   IO_POWER_ON_PIN
//#define TECO_MICROPHONE_ADC 0 // PF0
//#define TECO_PRESSURE_ADC   1 // PF1
//#define TECO_LIGHT_ADC      7 // PF7, if extra light  sensor connected 
//#define TECO_EXTRA_ADC      4 // PF4, if extra analog sensor connected 
//#define TECO_ADXL_X_PORT    PORTE   // PE6
//#define TECO_ADXL_X_PIN     PINE
//#define TECO_ADXL_X_BIT     6
//#define TECO_ADXL_Y_PORT    PORTB   // PB4
//#define TECO_ADXL_Y_PIN     PINB
//#define TECO_ADXL_Y_BIT     4
//#define TECO_ADXL_Z_PORT    PORTF   // PF4, if extra ADXL added
//#define TECO_ADXL_Z_PIN     PINF
//#define TECO_ADXL_Z_BIT     4
//#define TECO_SCL_PORT       PORTD   // PD0
//#define TECO_SCL_PIN        PIND
//#define TECO_SCL_BIT        0
//#define TECO_SDA_PORT       PORTD   // PD1
//#define TECO_SDA_PIN        PIND   
//#define TECO_SDA_BIT        1


