/*
 * Copyright (C) 2000-2006 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch/
 *
 */

/**
 * @file bt_defs.h
 * 
 * \author Mathias Payer <payerm@student.ethz.ch> 
 * 
 * \date 2004/04/14 21:37:09
 * 
 * @brief Defines general structs and variables
 */

#ifndef _BT_BT_DEFS_H_
#define _BT_BT_DEFS_H_

#define HCI_TRANSPORT UART

#define NUT_DEFAULT_STACKSIZE 2048

#include <sys/device.h>
#include <sys/mutex.h>

#include <hardware/btn-hardware.h>
#include <bt/bt_semaphore.h>
#include <bt/bt_hci_defs.h>
#include <stdio.h>

/**
 * @struct bt_hci_uart
 * This struct is used when the uart-transport is used.
 * In the struct btstack the transport ptr points to this struct
 * @see struct btstack
*/
struct bt_hci_uart {
    NUTDEVICE *hwdev;                   /**< Ptr. to the Hardware Device */
    FILE *uart;                     /**< File-Descriptor of the open device */
    struct bt_semaphore sem_write;      /**< Semaphore to prevent concurrent writes to uart */
#if (BT_RADIO_TYPE == ZEEVO)
    u_char reset_pending;           /**< Indicates HCI RESET CMD is processed */
#endif
};

#ifdef BT_PKT_STATS
/**
 * This struct is used for pkt statistics. There are counters for:
 *  - total number of received pkts
 *  - received pkts that have been discarded (buffer overrun)
 *  - sent pkts
 */
struct bt_pkt_stat {
    struct {
        u_short pkts;
        u_long bytes;
    } rx, rx_lost, tx;
};
#endif

/** 
 * Callbacks for the acl data
 */
#define HCI_ACL_CB struct bt_hci_pkt_acl *(*hci_acl_cb) (void *arg, struct bt_hci_pkt_acl *pkt, bt_hci_con_handle_t con_handle, u_char pb_flag, u_char bc_flag, u_short len, u_long t_arrive)

/**
 * \brief Definition of the acl callback registration function
 */
#define HCI_ACL_CB_REGISTRATION \
	void (*acl_cb_reg)(struct btstack* stack, HCI_ACL_CB, void* cb_arg)

/**
 *  Callbacks for the sco data  
 */
#define HCI_SCO_CB struct bt_hci_pkt_sco *(*hci_sco_cb) (void *arg, struct bt_hci_pkt_sco *pkt, bt_hci_con_handle_t con_handle, u_char len)

/**
 * Indicates that the role of this device is master
 */
#define BT_HCI_MY_ROLE_MASTER 0

/**
 * Indicates that the role of this device is slave
 */
#define BT_HCI_MY_ROLE_SLAVE  1

/**
 * Connection type
 */
#define BT_HCI_CONNECTION     2

/**
 * Disconnection type
 */
#define BT_HCI_DISCONNECTION  3

/**
 * Connection opened active, means this device triggered connection request
 */
#define BT_HCI_CONNECTION_ACTIVE    0

/**
 * Connection opened passive, means other device triggered connection request
 */
#define BT_HCI_CONNECTION_PASSIVE   1

/** 
 * @brief Callback for connection table changes.
 * 
 * To observe any connection, disconnection or role changes the application
 * can register a callback function #HCI_CON_TABLE_CB that will be called with
 * two arguments: type of change, extra info for some changes and the changed connection
 * It is important that the callback returns very fast!\n
 * \n
 * For #BT_HCI_CONNECTION details tells, if connection was opend by us 
 * #BT_HCI_CONNECTION_ACTIVE or #BT_HCI_CONNECTION_PASSIVE\n
 * For #BT_HCI_DISCONNECTION, details tells the disconnect reason
 *  
 */
#define HCI_CON_TABLE_CB \
	void (*con_table_cb) (u_char type, u_char detail, bt_hci_con_handle_t con_handle, void* cb_arg)

/**
 * \brief Definition of the con table change callback registration function
 */
#define HCI_CON_TABLE_CB_REGISTRATION \
	void (*con_table_cb_reg)(struct btstack* stack, HCI_CON_TABLE_CB, void* cb_arg)
	
/**
 *  @brief Callback to filter connection requests. 
 * 
 *  This function is called whenever a connection request event is received from
 *  the controller. Connections can be accepted by using #bt_hci_accept_connection_request
 *  or rejected by calling #bt_hci_reject_connection_request.
 * 
 *  Note: due to the current implementation of the bt-stack, connection requests must not
 *  be handled in the connection request callback - they have to be handled in a separate
 *  context.
 * 
 *  Note: Connection requests are turned off by default (i.e. auto-acception
 *  of incoming connection requests is turned on). To change this policy, use
 *  #bt_hci_set_event_filter.
 * 
 *  For details, refer to the documentation of the Connection Request Event in the 
 *  BT spec.
 * 
 *  @param addr   The address of the requesting device
 *  @param cod    The class of device of the requesting device
 *  @param type   Requested link type (0x00 for SCO, 0x01 for ACL, ...)
 *  @param arg    pointer to an optional argument that has been registered
 */
#define HCI_CON_RQST_CB void(*con_rqst_cb) (bt_addr_t addr, u_long cod, u_char type, void* arg)

/** 
 * The number of wait queues for synchronous running events, after completition 
 * these threads get woken up. Min: 8, max: 254.\n
 * Keep in mind that at most #NR_WAIT_QUEUES commands can be running at a time, 
 * even if they are called asynchronous! 
 */
#define BT_HCI_NR_WAIT_QUEUES 16

/*
 * Index of non existing entry in wait queue
 */
#define BT_HCI_WAIT_QUEUE_NONE 0xff

/** 
 * @brief returns the 2nd high byte of a u_short. 
 */
#define BTN_HI2(A) ((u_char)((A)>>16)&0xFF)

/** 
 * @brief returns the high byte of a u_short. 
 */
#define BTN_HI(A) ((u_char)((A)>>8)&0xFF)

/** 
 * @brief returns the low byte of a u_short. 
 */
#define BTN_LO(A) ((u_char)((A)&0xFF))

/**
 * @brief Default pin code for the BTnode
 */
#define BTN_DEFAULT_PIN "1234"

struct event_mask_high_low {
    u_long mask_high;  
    u_long mask_low;
};

struct host_buffer_len_nr {
    u_short acl_max_len;
    u_char sco_max_len;
    u_short nr_acl_pkts;
    u_short nr_sco_pkts;
};

struct param_list {
    u_char * baudrate; // u_char rate
    u_short * connection_accept_timeout; // u_short time
    u_short * default_link_policy_settings; // u_short settings
    u_char * event_filter; // different handling
    struct event_mask_high_low * event_mask; // u_long mask_high, u_long mask_low
    u_char * flowcontrol; // u_char type
    struct host_buffer_len_nr * host_buffer_size; // u_short acl_max_len, u_char sco_max_len, u_short nr_acl_pkts, u_short nr_sco_pkts
    u_char * inquiry_mode; // u_char mode
    u_char * inquiry_scan_type; // u_char mode
    u_long * local_cod; // u_long cod
    u_char * local_name; // u_char * name
    u_short * page_timeout; // u_short time
    u_char * pin_type; // u_char type
    u_char * scan_enable; // u_char type
};

/**
 * @struct btstack
 * @brief Represents one entity of a running stack (more possible)
 * Keeps all data for one stack, like devices, buffers, states, ...
 * @see struct bt_hci_uart for definition of uart-transport.
*/
struct btstack {
#if HCI_TRANSPORT == UART
    struct bt_hci_uart transport;       /**< UART-Transport definitions */
#else
    void transport;                     /**< USB-transport definitions (intentionally void -> error, because not yet implemented) */
#endif
    struct bt_semaphore nr_hci_cmds;    /**< Number of hci cmds we can send to the controller until it runs out of buffer */
    struct bt_semaphore hci_cmd_queue;  /**< Semaphore for the waiting commands. After one command is sent, the next thread is woken up.*/
    struct bt_hci_pkt_cmd cmd;          /**< Only one command at a time */
    struct bt_hci_pkt_evt evt;          /**< Only one event at a time */

    void *acl_data;                     /**< acl data (state of the acl-layer aka l2cap) */
    struct bt_hci_pkt_acl *acl_pkt;     /**< Pointer to the acl-buffer (local to the hci-layer) */
    struct bt_hci_pkt_acl* acl_self;	/**< Pointer to acl-buffer used for self-delivery */
    u_short acl_pkt_types;
    struct bt_hci_pkt_acl acl_init;     /**< Initial acl pkt. from this time on, the stack always gets one from the higher level */
    struct bt_semaphore nr_acl_pkts;    /**< Semaphore for the max. number of acl pkts the bt-controller can handle */
    u_short acl_max_len;                /**< Max. length of an acl-data-pkt the bt-controller will ever send */
    u_short host_acl_len;
    HCI_ACL_CB;                         /**< Callback for acl-data pkts */
    void *hci_acl_cb_arg;               /**< argument that will be passed to the callback */
    struct bt_semaphore acl_write;		/**< Semaphore for writing ACL data */
	bt_hci_con_handle_t acl_write_hdl;  /**< Currently locked ACL connection */
	u_short acl_write_len;				/**< Amount of data that is going to be sent over the currently locked ACL connection */
	u_short acl_written;				/**< Amount of data written to the currentely locked ACL connection */
	
    void *sco_data;                     /**< sco data (state of the sco-layer) */
    struct bt_hci_pkt_sco *sco_pkt;     /**< Pointer to the sco-buffer (local to the hci-layer) */
    struct bt_hci_pkt_sco sco_init;     /**< Initial sco pkt. from this time on, the stack always gets one from the higher level */

    struct bt_semaphore nr_sco_pkts;    /**< Semaphore for the max. number of sco pkts, the bt-controller can handle */
    u_char sco_max_len;                 /**< Max. length of a sco-data-pkt the bt-controller will ever send */
    HCI_SCO_CB;                         /**< Callback for sco-data pkts */

    u_char reset;                       /**< Reset flag */
    NUTTHREADINFO *hw_reset;            /**< Thread context from hw_reset caller*/
    struct bt_semaphore inquiry;

    struct bt_hci_cmd_response *waitqueue[BT_HCI_NR_WAIT_QUEUES];       /**< Keeps an eye on running pkts */
    u_char latest_cmd;                  /**< Index of cmd in waitqueue waiting for Command Status Event, 0xff == none */
    struct bt_semaphore single_cmd;     /**< Ensure at most one Command is sent without a received Command Status Event */

    u_char conn_request;                /**< Flag to indicate if a conn. request is pending (handled by the main thread) */
    u_char pin_request;                 /**< Flag to indicate if a pincode request is pending (handled by the main thread) */
    u_char *pin_code;                   /**< Pin Code */
    HCI_CON_TABLE_CB;                   /**< Callback for changes in the connection-table */
    void* con_change_cb_arg;
    struct bt_hci_connection connection[BT_HCI_MAX_NUM_CON];            /**< Keeps track on open connections */

    u_char parser_state;
    u_char parser_bytes_to_read;
    u_char parser_bytes_read;

    HCI_CON_RQST_CB;/**< Callback to filter out connection requests */
    void * con_rqst_cb_arg;  /**< custom argument for accept filter callback*/
#ifdef BT_PKT_STATS    
    struct bt_pkt_stat acl_pkt_stat;
#endif
    bt_addr_t bt_addr;
    struct param_list params;
};
#endif
