/*
 * Copyright (C) 2000-2004 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch/
 *
 */
 
/**
 * @file bt_hci_cmds.h
 *
 * \author Mathias Payer <payerm@student.ethz.ch>
 *
 * \date 2004/04/18 22:22:09
 * 
 * @brief HCI commands (definition)
 * 
 */

#include <bt/bt_hci_api.h>
#include <bt/bt_hci_defs.h>
#include <bt/bt_defs.h>

/**
 * Use this define instead of a pointer to a response struct to proccess
 * a bluetooth stack command call synchronously.
 */
#define BT_HCI_SYNC 0x0
#define SET_OGF_OCF(ogf,ocf) ((ocf<<8)|(ogf<<2))



/*******************************************************************************
 ###############################################################################
 * LinkControl HCI Commands (sync & async)
 ###############################################################################
 ******************************************************************************/
/*! \name LinkControl HCI Commands (sync & async)
 * Tries to send a #HCI_COMMAND_DATA_PACKET, the OpcodeGroup is #HCI_OGF_LINK_CONTROL
 */


//@{
/*******************************************************************************
 * bt_hci_inquiry
 ******************************************************************************/
/**
 * @brief Does an inquiry and looks for other bluetooth devices near by.
 *
 * This function triggers an inquiry with max_time in seconds, a pointer
 * to an array of type struct bt_hci_inquiry_result to fill in the found devices
 * and a maximal number of devices number. The event will be set as soon as
 * number found or the timeout max_time reached!\n 
 * The result will be stored in the array res.
 *
 * Example:
\verbatim
	struct bt_hci_inquiry_result inqres[2];
	retval = bt_hci_inquiry(&stack, BT_HCI_SYNC, 0x03, 0x02, inqres);
\endverbatim
 * (This inquiry runs for 3*1.28 seconds (max.) and looks for at most 2 devs.)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or BT_HCI_SYNC
 * @param time Time to wait for the inquiry to complete, min: 0x00, max: 0x30, the actual waiting time is x*1.28 seconds
 * @param number Estimed number of deviced to find, min: 0x01, max: 0xFF
 * @param res Pointer to the array were the discovered devices will be stored (has to be of size number!)
 *
 * @return the # of found devices if the call was sync or possible errors:
 * - #BT_ERR_BT_MODULE
 */
long bt_hci_inquiry(struct btstack *stack, struct bt_hci_cmd_response *response, u_char time, u_char number, struct bt_hci_inquiry_result *res);

long bt_hci_inquiry_cancel(struct btstack *stack, struct bt_hci_cmd_response *response);

/**
 * After the connection completes, this device will be master 
 */
#define BT_HCI_ROLE_SWITCH_MASTER	0x00	

/**
 * After the connection completes, this device may be slave (but needn't be!) 
 */
#define BT_HCI_ROLE_SWITCH_CHANGE	0x01	

/*******************************************************************************
 * bt_hci_create_connection
 ******************************************************************************/
/**
 * @brief This function opens a connection to a bluetooth device.
 * 
 * Tries to open a hci-connection to the given address.
 *
 * This function opens a connection to a bluetooth device with address addr
 * and a flag indicating if role switch allowed (means this device might be 
 * slave after the connection has been opened) or not (this device will be 
 * master).\n
 * With the con_response the application can observe explicitly the opening of 
 * THIS connection!\n
 * To speed up the connection establishment the clock offset and the to the device
 * retrieved during inquiry can be passed.
 *
 * To use this function as a synchronous call, pass a #BT_HCI_SYNC as name_response.
 *
 * If there is already an open connection to this address, it immediately retures with the connection handle
 * 
 * This function can also correctly handle multiple connection requests to the same device and/or
 * connection requests to devices from whom an incoming connection request is pending
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param addr Bluetooth address to connect to
 * @param packet_type The packet-types to accept. Multiple values can be or'ed (|)
 * @param page_scan_rep_mode (found during inquiry) (pass 0 if unknown)
 * @param clock_offset Offset to this device (found during inquiry). pass 0 if unknown
 * @param allow_role_switch Indicates if a roleswitch is possible (#BT_HCI_ROLE_SWITCH_MASTER or #BT_HCI_ROLE_SWITCH_CHANGE)
 *
 * @return The connection-handle if the call was sync or possible errors:
 * - #BT_ERR_BT_MODULE
 * - #BT_ERR_CON_EXIST
 * - #BT_ERR_CON_PEND_ACTIVE
 * - #BT_ERR_CON_PEND_PASSIVE
 */
long bt_hci_create_connection(struct btstack *stack, struct bt_hci_cmd_response *response, bt_addr_t addr, u_short packet_type,
u_char page_scan_rep_mode, u_short clock_offset, u_char allow_role_switch);

long bt_hci_create_connection_cancel(struct btstack *stack, struct bt_hci_cmd_response *response, bt_addr_t addr);


/*******************************************************************************
 * bt_hci_change_con_pkt_type
 ******************************************************************************/
/**
 * @brief To change the connection packet type of a certain connection.
 *
 * With the response the application can observe explicitly the change of the packet type.
 *
 * To use this function as a synchronous call, pass a #BT_HCI_SYNC as name_response.
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle The con. handle to change the packet type
 * @param packet_type:
 * - #BT_HCI_PACKET_TYPE_DM1
 * - #BT_HCI_PACKET_TYPE_DH1
 * - #BT_HCI_PACKET_TYPE_DM3
 * - #BT_HCI_PACKET_TYPE_DH3
 * - #BT_HCI_PACKET_TYPE_DM5
 * - #BT_HCI_PACKET_TYPE_DH5
 * @return Packet type that is set now if the call was sync or possible errors:
 * - #BT_ERR_BT_MODULE
 * - #BT_ERR_NO_CON
 */
long bt_hci_change_con_pkt_type(struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t con_handle, u_short packet_type);


/**
 *  We disconnect/were disconnected because of an authentication failure 
 */
#define BT_HCI_DISCONNECT_AUTH_FAILURE		0x05	

/**
 * We disconnect/were disconnected because the user terminated the connection 
 */
#define BT_HCI_DISCONNECT_USER_ENDED		  0x13	

/**
 *  We disconnect/were disconnected because of low resources 
 */
#define BT_HCI_DISCONNECT_LOW_RESOURCES		0x14	

/**
 *  We disconnect/were disconnected because of a power down 
 */
#define BT_HCI_DISCONNECT_POWER_DOWN		  0x15	

/**
 *  We disconnect/were disconnected because of an unsupported remote feature 
 */
#define BT_HCI_DISCONNECT_UNSUPP_FEATURE	0x1A	

/**
 *  We disconnect/were disconnected because the pairing with unit key was not supported 
 */
#define BT_HCI_DISCONNECT_PAIRING_UNSUPP	0x29	

/*******************************************************************************
 * bt_hci_disconnect
 ******************************************************************************/
/**
 * @brief Tries to disconnect a hci-connection with the given handle.
 * 
 * With the con_response the application can observe explicitly the opening of 
 * THIS connection!
 *
 * To use this function as a synchronous call, pass a #BT_HCI_SYNC as name_response.
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle The con. handle to disconnect
 * @param reason The reason for the disconnect:
 * - #BT_HCI_DISCONNECT_AUTH_FAILURE
 * - #BT_HCI_DISCONNECT_USER_ENDED
 * - #BT_HCI_DISCONNECT_LOW_RESOURCES
 * - #BT_HCI_DISCONNECT_POWER_DOWN
 * - #BT_HCI_DISCONNECT_UNSUPP_FEATURE
 * - #BT_HCI_DISCONNECT_PAIRING_UNSUPP
 *
 * @return 0 if everything is fine and the call was sync or possible errors:
 * - #BT_ERR_BT_MODULE
 * - #BT_ERR_NO_CON
 */
long bt_hci_disconnect(struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t con_handle, u_char reason);

/*******************************************************************************
 * bt_hci_accept_connection_request
 ******************************************************************************/
/**
 * Accepts a connection request from a remote device
 *
 * This function may only be used by the stack internally!
 *
 * @param stack The bt-stack to use
 * @param addr BT-Addr to look for
 * @param role Master or slave ( #BT_HCI_MY_ROLE_MASTER | #BT_HCI_MY_ROLE_SLAVE )
 *
 * @return 0 or possible errors:
 * - #BT_ERR_BT_MODULE
 */
long bt_hci_accept_connection_request(struct btstack *stack, bt_addr_t addr, u_char role);

/*******************************************************************************
 * bt_hci_link_key_request_reply
 ******************************************************************************/
/**
 * @brief The Link_Key_Request_Reply command is used to reply to a Link Key 
 * Request event from the Controller, and specifies the Link Key stored on the 
 * Host to be used as the link key for the connection with the other Bluetooth 
 * Device specified by BD_ADDR. The Link Key Request event will be generated 
 * when the Controller needs a Link Key for a connection.
 * 
 * @param stack The bt-stack to use
 * @param con_handle The connection handle to use
 * @param addr BD_ADDR for this pin code
 * @param link_key
 *
 * @return BD_ADDR that completed the pin code request
 */
long bt_hci_link_key_request_reply(struct btstack *stack, bt_hci_con_handle_t con_handle, bt_addr_t addr, u_char *link_key);

/*******************************************************************************
 * bt_hci_link_key_request_negative_reply
 ******************************************************************************/
/**
 * @brief The Link_Key_Request_Negative_Reply command is used to reply to a Link
 * Key Request event from the Controller if the Host does not have a stored Link
 * Key for the connection with the other Bluetooth Device specified by BD_ADDR.
 * 
 * @param stack The bt-stack to use
 * @param addr BD_ADDR for this pin code
 *
 * @return 0 on success
 */
long bt_hci_link_key_request_negative_reply(struct btstack *stack, bt_addr_t addr);

/*******************************************************************************
 * bt_hci_pin_code_request_reply
 ******************************************************************************/
/**
 * @brief The PIN_Code_Request_Reply command is used to reply to a PIN Code 
 * request event from the Controller, and specifies the PIN code to use for a 
 * connection. The PIN Code Request event will be generated when a connection 
 * with remote initiating device has requested pairing.
 * 
 * @param stack The bt-stack to use
 * @param con_handle The connection handle to use
 * @param addr BD_ADDR for this pin code
 * @param pin_length
 * @param pin_code
 *
 * @return BD_ADDR that completed the pin code request
 */
long bt_hci_pin_code_request_reply(struct btstack *stack, bt_hci_con_handle_t con_handle, bt_addr_t addr, u_char pin_length, u_char *pin_code);

/*******************************************************************************
 * bt_hci_authentication requested
 ******************************************************************************/
/**
 * @brief The Authentication_Requested command is used to try to authenticate the 
 * remote device associated with the specified Connection Handle. On an 
 * authentication failure, the Controller or Link Manager shall not automatically 
 * detach the link. The Host is responsible for issuing a Disconnect command to 
 * terminate the link if the action is appropriate.
 * 
 * @param stack The bt-stack to use
 * @param con_handle The connection handle to use
 */
long bt_hci_authentication_requested(struct btstack *stack, bt_hci_con_handle_t con_handle);

/*******************************************************************************
 * bt_hci_remote_name_request
 ******************************************************************************/
/**
 * @brief Looks for the human-readable name of a given bd-address.
 * 
 * Max. length of name is 248 chars.
 *
 * If the returned length is larger than the passed maxlen, then one knows, that 
 * the name did not fit into the array.
 *
 * This command is only available in a synchronized version!
 *
 * @param stack The bt-stack to use
 * @param addr BT-Addr to look for
 * @param clock_offset Offset to this device (found during inquiry). pass 0 if unknown
 * @param page_scan_repetition_mode also found during inquiry, pass 0 if unsure
 * @param name The name of this device, '\\0' terminated
 * @param maxlen The length of the string passed in the name parameter (nr. of allocated bytes)
 *
 * @return the len of the name if everything is find and the call was sync or possible errors:
 * - #BT_ERR_BT_MODULE
 */
long bt_hci_remote_name_request(struct btstack *stack, bt_addr_t addr, u_short clock_offset, u_char page_scan_repetition_mode, char *name, u_char maxlen);

/*******************************************************************************
* bt_hci_read_clock_offset
******************************************************************************/
/**
 * Read clock offset to remove device
 *
 * The clock offset is specified as Bit 16-2 of CLKslabe-CLKmaster (MOD 32768)
 * which means that the clock difference is always possitive and has a 1.25 ms
 * resolution.
 * 
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle The con. handle of remote device
 *
 * @return clock offset or possible errors:
 * - #BT_ERR_NO_CON
 * - #BT_ERR_BT_MODULE
 */
long bt_hci_read_clock_offset(struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t app_con_handle);

//@}

/*******************************************************************************
 ###############################################################################
 * LinkPolicy HCI Commands
 ###############################################################################
 ******************************************************************************/

/*! \name LinkPolicy HCI Commands (sync & async)
 * 
 */


//@{

/*******************************************************************************
 * bt_hci_role_discovery
 ******************************************************************************/
/**
 * @brief Returns the Role of the specified con_handle.
 *
 * Do not confuse this command with bt_hci_local_role_discovery(...) that
 * gets the information only from the entry in the connection table!
 * 
 * This cmd actually asks the bluetooth module.
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle The pointer where the result will be stored
 *
 * @return #BT_HCI_MY_ROLE_MASTER, #BT_HCI_MY_ROLE_SLAVE if everything was ok 
 * and the call was sync or possible errors:
 * - #BT_ERR_NO_CON
 */
long bt_hci_role_discovery(struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t con_handle);


/*******************************************************************************
 * bt_hci_role_set
 ******************************************************************************/
/**
 * @brief Sends the role change request to the connected device with con_handle.
 * 
 * This request can be observed explicitly with passing a role_response.
 * To use this function as a synchronous call, pass #BT_HCI_SYNC as role_response and 
 * and the new role will be passed as return value.
 * Otherwise the new role will be set in the role_response struct!
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle The pointer where the result will be stored
 * @param role The new role of this device: #BT_HCI_MY_ROLE_MASTER or #BT_HCI_MY_ROLE_SLAVE
 *
 * @return the new role if the call was sync or possible errors:
 * - #BT_ERR_NO_CON
 * - \ref Bt_Error_Codes Nr. 0x21
 */
long bt_hci_role_set(struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t con_handle, u_char role);

/** 
 * All options disabled 
 */
#define BT_HCI_LINK_POLICY_ALL_DISABLED	0x0000	
/** 
 * Enables role switch 
 */
#define BT_HCI_LINK_POLICY_ROLE_SWITCH	  0x0001
/** 
 * Enables hold mode 
 */	
#define BT_HCI_LINK_POLICY_HOLD_MODE	   	0x0002	
/** 
 * Enables sniff mode 
 */
#define BT_HCI_LINK_POLICY_SNIFF_MODE		0x0004	
/** 
 * Enables park state 
 */
#define BT_HCI_LINK_POLICY_PARK_STATE		0x0008

/*******************************************************************************
 * bt_hci_read_link_policy_settings
 ******************************************************************************/
/**
 * @brief Reads the link policy settings for a given connection.
 *
 * Multiple settings may be or'ed (|)!\n
 * Possible Settings:\n
 * - #BT_HCI_LINK_POLICY_ALL_DISABLED All options disabled
 * - #BT_HCI_LINK_POLICY_ROLE_SWITCH  Enables role switch
 * - #BT_HCI_LINK_POLICY_HOLD_MODE  Enables hold mode
 * - #BT_HCI_LINK_POLICY_SNIFF_MODE Enables sniff mode
 * - #BT_HCI_LINK_POLICY_PARK_STATE Enables park state
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle the application connection handle
 *
 * @return setting or possible errors:
 * - #BT_ERR_NO_CON
 * - \ref Bt_Error_Codes
 */
long bt_hci_read_link_policy_settings(struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t con_handle);

/*******************************************************************************
 * bt_hci_write_link_policy_settings
 ******************************************************************************/
/**
 * @brief Sets the new link policy settings for a given connection.
 *
 * Multiple settings may be or'ed (|)!\n
 * Possible Settings:\n
 * - #BT_HCI_LINK_POLICY_ALL_DISABLED All options disabled
 * - #BT_HCI_LINK_POLICY_ROLE_SWITCH  Enables role switch
 * - #BT_HCI_LINK_POLICY_HOLD_MODE  Enables hold mode
 * - #BT_HCI_LINK_POLICY_SNIFF_MODE Enables sniff mode
 * - #BT_HCI_LINK_POLICY_PARK_STATE Enables park state
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle the application connection handle
 * @param settings The new settings
 *
 * @return 0 or possible errors:
 * - #BT_ERR_NO_CON
 * - \ref Bt_Error_Codes
 */
long bt_hci_write_link_policy_settings(struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t con_handle, u_short settings);

/*******************************************************************************
 * bt_hci_read_default_link_policy_settings
 ******************************************************************************/
/**
 * @brief Reads the default link policy settings for all connections.
 *
 * Multiple settings may be or'ed!\n
 * Possible Settings:
 * - #BT_HCI_LINK_POLICY_ALL_DISABLED   All options disabled
 * - #BT_HCI_LINK_POLICY_ROLE_SWITCH    Enables role switch
 * - #BT_HCI_LINK_POLICY_HOLD_MODE      Enables hold mode
 * - #BT_HCI_LINK_POLICY_SNIFF_MODE Enables sniff mode
 * - #BT_HCI_LINK_POLICY_PARK_STATE Enables park state
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 *
 * @return setting or possible errors:
 * - #BT_ERR_NO_CON
 * - \ref Bt_Error_Codes
 */
long bt_hci_read_default_link_policy_settings(struct btstack *stack, struct bt_hci_cmd_response *response); 

/*******************************************************************************
 * bt_hci_write_default_link_policy_settings
 ******************************************************************************/
/**
 * @brief Sets the new link policy settings for all connections.
 *
 * Multiple settings may be or'ed!\n
 * Possible Settings:
 * - #BT_HCI_LINK_POLICY_ALL_DISABLED	All options disabled
 * - #BT_HCI_LINK_POLICY_ROLE_SWITCH	Enables role switch
 * - #BT_HCI_LINK_POLICY_HOLD_MODE		Enables hold mode
 * - #BT_HCI_LINK_POLICY_SNIFF_MODE	Enables sniff mode
 * - #BT_HCI_LINK_POLICY_PARK_STATE	Enables park state
 *
 * remark: ROLE_SWITCH only works for slave role, somehow master is not allowed
 * to change the role at the first time. after one role switch, master and
 * slave are able to change the role!
 * 
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param settings The new default settings
 *
 * @return 0 or possible errors:
 * - #BT_ERR_NO_CON
 * - \ref Bt_Error_Codes
 */
long bt_hci_write_default_link_policy_settings(struct btstack *stack, struct bt_hci_cmd_response *response, u_short settings);

/**
 * @brief Sets hold mode for a given connection.
 *
 * remark: the controller will return the actual hold interval in the interval
 * parameter of the mode change event.
 * 
 * mandatory range for interval: 0x0014-0x8000
 * 
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle the application connection handle
 * @param max_interval maximum acceptable number of baseband slots to wait in hold mode (range: 0x0002-0xfffe, only even values are valid)
 * @param min_interval minimum acceptable number of baseband slots to wait in hold mode (range: 0x0002-0xff00, only even values are valid)
 *
 * @return 0 or possible errors:
 * - #BT_ERR_NO_CON
 * - \ref Bt_Error_Codes
 */
long bt_hci_hold_mode(struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t app_con_handle, u_short max_interval, u_short min_interval);

/**
 * @brief Sets sniff mode for a given connection.
 *
 * remark: the controller will return the actual sniff interval in the interval
 * parameter of the mode change event.
 * 
 * it is possible for the master to transmit data to a slave without exiting sniff mode!
 * 
 * limitations by Zeevo ZV4002:
 * - the Zeevo device can only sniff in one piconet. hence if a sniff link already exists in another piconet, the new proposed sniff will be rejected.
 *.- if the Zeevo device has multiple sniffed links (all slaves), they must have the same interval.
 * 
 * the max_interval shall be less than the link supervision timeout configuration parameter.
 * 
 * mandatory range for interval: 0x0006-0x0540
 * mandatory range for attempt: 0x0001-(interval/2)
 * mandatory range for timout: 0x0000-0x0028
 * 
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle the application connection handle
 * @param max_interval maximum acceptable number of baseband slots for sniff period (range: 0x0002-0xfffe, only even values are valid)
 * @param min_interval minimum acceptable number of baseband slots for sniff period (range: 0x0002-0xfffe, only even values are valid)
 * @param attempt number of baseband receive slots for sniff attempt (range: 0x0001-0x7fff)
 * @param timeout number of baseband receive slots for sniff timeout (range: 0x0001-0x7fff)
 * 
 * @return 0 or possible errors:
 * - #BT_ERR_NO_CON
 * - \ref Bt_Error_Codes
 */
long bt_hci_sniff_mode(struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t app_con_handle, u_short max_interval, u_short min_interval, u_short attempt, u_short timeout);

/**
 * @brief Exits sniff mode for a given connection.
 *
 * @param stack The bt-stack to use
 * @param con_handle the application connection handle
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * 
 * @return 0 or possible errors:
 * - #BT_ERR_NO_CON
 * - \ref Bt_Error_Codes
 */
long bt_hci_exit_sniff_mode(struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t app_con_handle);

/**
 * @brief Sets park state for a given connection.
 *
 * remark: the controller will return the actual beacon interval in the interval
 * parameter of the mode change event.
 * 
 * it is possible for the controller to do an automatic unpark to transmit data and then park the connection again! the optional feature of automatic unpark/park can also be used for link supervision.
 * - Zeevo ZV4002: no support
 * 
 * so its possible to have until 255 parked slaves in one piconet.
 * - Zeevo ZV4002: approximately 10 parked slaves (has to be tested/verified)
 * 
 * mandatory range for beacon interval: 0x000e-0x1000
 * 
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle the application connection handle
 * @param max_interval maximum acceptable number of baseband slots for beacon period (range: 0x000E-0xfffe, only even values are valid)
 * @param min_interval minimum acceptable number of baseband slots for beacon period (range: 0x000E-0xfffe, only even values are valid)
 * 
 * @return 0 or possible errors:
 * - #BT_ERR_NO_CON
 * - \ref Bt_Error_Codes
 */
long bt_hci_park_state(struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t app_con_handle, u_short beacon_max_interval, u_short beacon_min_interval);

/**
 * @brief Exits park state for a given connection.
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle the application connection handle
 * 
 * @return 0 or possible errors:
 * - #BT_ERR_NO_CON
 * - \ref Bt_Error_Codes
 */
long bt_hci_exit_park_state(struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t app_con_handle);
//@}


/*******************************************************************************
 ###############################################################################
 * HostControl HCI Commands
 ###############################################################################
 ******************************************************************************/
/*! \name HostControl HCI Commands (sync & async)
 * Tries to send a #HCI_COMMAND_DATA_PACKET, the OpcodeGroup is #HCI_OGF_HOST_CONTROL
 */


//@{

#define BT_HCI_DEFAULT_EVENT_MASK_HIGH  0x00001FFF
#define BT_HCI_DEFAULT_EVENT_MASK_LOW   0xFFFFFFFF
#define BT_HCI_EVENT_LINK_KEY_REQUEST   0x00400000
/*******************************************************************************
 * bt_hci_set_event_mask
 ******************************************************************************/
/**
 * @brief Sets the HCI event mask.
 * 
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param mask_high The higher 32 bit of the event mask
 * @param mask_low The lower 32 bit of the event mask
 *
 * @return 0 on success
 */
long bt_hci_set_event_mask(struct btstack *stack, struct bt_hci_cmd_response *response, u_long mask_high, u_long mask_low);


/*******************************************************************************
 * bt_hci_reset
 ******************************************************************************/
/**
 * @brief Resets the controller (but not the stack!), no parameters needed.
 * 
 * \attention You have to reset the stack manually before!\n
 * The hci-cmd itself takes no parameters!
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_reset(struct btstack *stack, struct bt_hci_cmd_response *response);


/*******************************************************************************
 * bt_hci_set_event_filter
 ******************************************************************************/
/**
 * @brief Sets the HCI event filter.
 * 
 *	HCI_OCF_HC_SET_EVENT_FILTER:\n
 *		The Set Event Filter command is used by the Host to specify different
 * 		event filters.
 *		This command looks like a little messed up - the number and size of
 *		the parameters depend on various previous parameters...\n
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param param OpenParameter-List, depends on cmd (see below)
\verbatim
 		first parameter: Filter type.
 		0x00 - No filter (clear all filter)
 		0x01 - Inquiry Filter
 			second parameter: inq. result filter condition:
 			0x00: return all events
 			0x01: filter on device class
 				third parameter: class of device (u_long)
 				forth parameter: device-mask (active bits, u_long)
 			0x02: show only one spec. device
 				third parameter: device (bt_addr_t)
 		0x02 - Connection Filter
 			second parameter: conn. setup filter type:
 			0x00: allow connections from all devices
 				third parameter: auto_accept_flag (u_char)
 				0x01 - no auto accept
 				0x02 - auto accept / no role switch
 				0x03 - auto accept & role switch
 			0x01: allow connect from device class
 				third parameter: class of device (u_long)
 				forth parameter: device-mask (active bits, u_long)
 				fifth parameter: auto_accept_flag (u_char)
 				0x01 - no auto accept
 				0x02 - auto accept / no role switch
 				0x03 - auto accept & role switch
 			0x02: show only one spec. device
 				third parameter: device (bt_addr_t)
 				forth parameter: auto_accept_flag (u_char)
 				0x01 - no auto accept
 				0x02 - auto accept / no role switch
 				0x03 - auto accept & role switch
\endverbatim
 * 
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 * 
 * Note: if setting up a connection filter and auto accept is turned off
 * (i.e. the auto-accept flag is set to 0x01), handling connection requests must
 * be done by the application itself. This has to be done by registering a
 * connection request callback using #bt_hci_register_con_request_cb.
 */
long bt_hci_set_event_filter(struct btstack *stack, struct bt_hci_cmd_response *response, u_char param, ...);
long _bt_hci_set_event_filter_bytearray(struct btstack* stack, struct bt_hci_cmd_response* response, u_char * byte_array);


#define BT_HCI_PIN_TYPE_VARIABLE	0x00	/**< Variable PIN-Type @see bt_hci_read_pin_type, bt_hci_write_pin_type */
#define BT_HCI_PIN_TYPE_FIXED		0x01	/**< Fixed PIN-Type @see bt_hci_read_pin_type, bt_hci_write_pin_type */
/*******************************************************************************
 * bt_hci_read_pin_type
 ******************************************************************************/
/**
 * @brief This parameter controls wether the PIN (for pairing) is fixed or variable
 * 
 * Possible values for type:
 * 	- #BT_HCI_PIN_TYPE_VARIABLE (variable pin)
 * 	- #BT_HCI_PIN_TYPE_FIXED (fixed pin)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 *
 * @return 0 (async), #BT_HCI_PIN_TYPE_VARIABLE, #BT_HCI_PIN_TYPE_FIXED or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_read_pin_type(struct btstack *stack, struct bt_hci_cmd_response *response);


/*******************************************************************************
 * bt_hci_write_pin_type
 ******************************************************************************/
/**
 * @brief  * This parameter controls wether the PIN (for pairing) is fixed or variable.
 * 
 * Possible values for type:
 * 	- #BT_HCI_PIN_TYPE_VARIABLE (variable pin)
 * 	- #BT_HCI_PIN_TYPE_FIXED (fixed pin)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param type #BT_HCI_PIN_TYPE_VARIABLE or #BT_HCI_PIN_TYPE_FIXED
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_write_pin_type(struct btstack *stack, struct bt_hci_cmd_response *response, u_char type);


/*******************************************************************************
 * bt_hci_write_local_name
 ******************************************************************************/
/**
 * @brief This cmd sets the local name of the bt-device referenced by the stack.
 * 
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param name The name of this device terminated with '\\0'
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_write_local_name(struct btstack *stack, struct bt_hci_cmd_response *response, u_char *name);


/*******************************************************************************
 * bt_hci_read_local_name
 ******************************************************************************/
/**
 * @brief This cmd gets the local name of the bt-device referenced by the stack.
 * 
 * Max. length of name is 248 chars.
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param name Pointer to a free char array for the local name
 * @param maxlen Length of the free char array
 *
 * @return len of name if everything is fine and the call was sync or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_read_local_name(struct btstack *stack, struct bt_hci_cmd_response *response, u_char *name, u_char maxlen);


/*******************************************************************************
 * bt_hci_read_connection_accept_timeout
 ******************************************************************************/
/**
 * @brief This reads the max. time the controller will wait for the stack after a
 * connection request event is sent.
 * 
 * If the stack does not respond in the given time, the connections is rejected.
 * The default value is 0x1F40 (=5.00 s)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 *
 * @return Waittime = time * 0.625msec (1 Baseband slot) range: 0x0001 - 0xB540
 *  if everything is fine and the call was sync or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_read_connection_accept_timeout(struct btstack *stack, struct bt_hci_cmd_response *response);


/*******************************************************************************
 * bt_hci_write_connection_accept_timeout
 ******************************************************************************/
/**
 * @brief This writes the max. time the controller will wait for the stack after a
 * connection request event is sent.
 * 
 * If the stack does not respond in the given time, the connections is rejected.
 * The default value is 0x1F40 (=5.00 s)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param time Waittime = time * 0.625msec (1 Baseband slot) range: 0x0001 - 0xB540
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_write_connection_accept_timeout(struct btstack *stack, struct bt_hci_cmd_response *response, u_short time);


/*******************************************************************************
 * bt_hci_read_page_timeout
 ******************************************************************************/
/**
 * @brief This reads the max. time the local link manager will wait for a baseband page
 * response from the remote device at a locally initiated connection attempt.
 * 
 * The default value is 0x2000 (=5.12 s)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param time 
 *
 * @return the read-page-timeout \n
 * Waittime = time * 0.625msec (1 Baseband slot) range: 0x0001 - 0xEFFF \n
 * if the call was sync or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_read_page_timeout(struct btstack *stack, struct bt_hci_cmd_response *response);


/*******************************************************************************
 * bt_hci_write_page_timeout
 ******************************************************************************/
/**
 * @brief This defines the max. time the local link manager will wait for a baseband page
 * response from the remote device at a locally initiated connection attempt.
 * 
 * The default value is 0x2000 (=5.12 s)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param time Waittime = time * 0.625msec (1 Baseband slot) range: 0x0001 - 0xEFFF
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_write_page_timeout(struct btstack *stack, struct bt_hci_cmd_response *response, u_short time);


#define BT_HCI_SCAN_MODE_OFF		0x00	/**< this device is hidden @see bt_hci_read_scan_enable, bt_hci_write_scan_enable */
#define BT_HCI_SCAN_MODE_INQUIRY	0x01	/**< only inquiry allowed @see bt_hci_read_scan_enable, bt_hci_write_scan_enable */
#define BT_HCI_SCAN_MODE_PAGE		0x02	/**< only page scans allowed @see bt_hci_read_scan_enable, bt_hci_write_scan_enable */
#define BT_HCI_SCAN_MODE_INQUIRY_PAGE	0x03	/**< both inquiry & page scans allowd @see bt_hci_read_scan_enable, bt_hci_write_scan_enable */
/*******************************************************************************
 * bt_hci_read_scan_enable
 ******************************************************************************/
/**
 * @brief This parameter controls wether or not the bt-device will periodically scan for
 * page attemps and/or inquiry requests from other devices.
 * 
 * Possible values for type:
 * 	- #BT_HCI_SCAN_MODE_OFF (this device is hidden)
 * 	- #BT_HCI_SCAN_MODE_INQUIRY (only inquiry allowed)
 *	- #BT_HCI_SCAN_MODE_PAGE (only page scans allowed)
 *	- #BT_HCI_SCAN_MODE_INQUIRY_PAGE (both inquiry & page scans allowd)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 *
 * @return the type or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_read_scan_enable(struct btstack *stack, struct bt_hci_cmd_response *response);


/*******************************************************************************
 * bt_hci_write_scan_enable
 ******************************************************************************/
/**
 * @brief This parameter controls wether or not the bt-device will periodically scan for
 * page attemps and/or inquiry requests from other devices.
 * 
 * Possible values for type:
 * 	- #BT_HCI_SCAN_MODE_OFF (this device is hidden)
 * 	- #BT_HCI_SCAN_MODE_INQUIRY (only inquiry allowed)
 *	- #BT_HCI_SCAN_MODE_PAGE (only page scans allowd)
 *	- #BT_HCI_SCAN_MODE_INQUIRY_PAGE (both inquiry & page scans allowd)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param type One of the possible values for type
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_write_scan_enable(struct btstack *stack, struct bt_hci_cmd_response *response, u_char type);


/*******************************************************************************
 * bt_hci_read_local_cod
 ******************************************************************************/
/**
 * @brief Reads the local class of device.
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * 
 * @return cod (lower 24 Bit) or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 */
long bt_hci_read_local_cod(struct btstack *stack, struct bt_hci_cmd_response *response);


/*******************************************************************************
 * bt_hci_write_local_cod
 ******************************************************************************/
/**
 * @brief Writes the local calss of device.
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param cod Pointer to the local class of device (lower 24 Bit)
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_write_local_cod(struct btstack *stack, struct bt_hci_cmd_response *response, u_long cod);


#define BT_HCI_FLOWCONTROL_OFF		0x00	/**< no flowcontrol @see bt_hci_set_flowcontrol */
#define BT_HCI_FLOWCONTROL_ACL		0x01	/**< flowcontrol for acl-pkts @see bt_hci_set_flowcontrol */
#define BT_HCI_FLOWCONTROL_SCO		0x02	/**< flowcontrol for sco-pkts @see bt_hci_set_flowcontrol */
#define BT_HCI_FLOWCONTROL_ACL_SCO	0x03	/**< flowcontrol for acl & sco @see bt_hci_set_flowcontrol */
/*******************************************************************************
 * bt_hci_set_flowcontrol
 ******************************************************************************/
/**
 * @brief This parameter sets the flow-control to the given type.
 * 
 * If the flow-control is active the host must send host_number_of_completed_pkts!
 *
 * The default is #BT_HCI_FLOWCONTROL_ACL, because SCO is not supported by the
 * bt-controller!
 *
 * \attention This command may NOT be called from an application !\n
 * It is internal for the stack and is only used after a reset to get these
 * informational parameters and must be called only if no connections are open!
 *
 * Possible values for type:
 * 	- #BT_HCI_FLOWCONTROL_OFF (no flowcontrol)
 * 	- #BT_HCI_FLOWCONTROL_ACL (flowcontrol for acl-pkts)
 *	- #BT_HCI_FLOWCONTROL_SCO (flowcontrol for sco-pkts)
 *	- #BT_HCI_FLOWCONTROL_ACL_SCO (flowcontrol for acl & sco)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param type One of the possible values for type
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_set_flowcontrol(struct btstack *stack, struct bt_hci_cmd_response *response, u_char type);


/*******************************************************************************
 * bt_hci_host_buffer_size
 ******************************************************************************/
/**
 * @brief This cmd sets the max. number of {acl|sco}-data-pkts and the max. size of
 * an {acl|sco}-data-pkt the bt-stack can handle.
 * 
 * According to the max. size the bt-controller will fragment all sco and acl
 * Data-pkts.
 *
 * \attention This command may NOT be called from an application !\n
 * It is internal for the stack and is called when the acl- and/or sco-layer is
 * initialized.\n
 * ACL and SCO-Layer must cooperate and set all params at once!\n
 * (With one call, both acl and sco max-size and number of buffers is set!)\n
 * This funtion MUST be called by the acl and/or sco layer to initialize the
 * bt-controller!
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param acl_max_len Max. length for any acl-data pkt
 * @param sco_max_len Max. length for any sco-data pkt
 * @param nr_acl_pkts Max. number of acl-buffers the stack can handle
 * @param nr_sco_pkts Max. number of sco-buffers the stack can handle
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_host_buffer_size(struct btstack *stack, struct bt_hci_cmd_response *response, u_short acl_max_len, u_char sco_max_len, u_short nr_acl_pkts, u_short nr_sco_pkts);


/*******************************************************************************
 * bt_hci_host_buffer_size
 ******************************************************************************/
/**
 * @brief This cmd tells the module that one or more acl buffer has been freed again.
 * 
 * (using the Host Number of Completed Packets Command)
 * this is used to prevent overflow on our side
 *
 *
 * @param stack The bt-stack to use
 * @param con_handle The con_handle where pkgs were completed
 * @param nr The number of completed packages
 *
 * @return 0 or possible errors:
 *  - #BT_ERR_BT_MODULE
 */
long bt_hci_acl_free_buffer(struct btstack *stack, bt_hci_con_handle_t con_handle, u_short nr);


/*******************************************************************************
 * bt_hci_read_link_supervision_timeout
 ******************************************************************************/
/**
 * @brief This cmd reads the supervision timeout (timeout that will be triggered
 * if the link is lost).

 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle The con_handle for which link supervision timeout is to be read
 *
 * @return link supervions timeout x 0.625 ms or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_NO_CON
 */
long bt_hci_read_link_supervision_timeout (struct btstack *stack, struct bt_hci_cmd_response *response, bt_hci_con_handle_t con_handle);


/*******************************************************************************
 * bt_hci_write_link_supervision_timeout
 ******************************************************************************/
/**
 * @brief This cmd writes the supervision timeout (timeout that will be triggered
 * if the link is lost).
 *
 * @param stack The bt-stack to use
 * @param con_handle The con_handle for which link supervision timeout is to be set
 * @param timeout The link supervision timeout x 0.625ms
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_NO_CON
 */
long bt_hci_write_link_supervision_timeout (struct btstack *stack, struct bt_hci_cmd_response *response,
                                           bt_hci_con_handle_t con_handle, u_short timeout);


#define BT_HCI_INQUIRY_MODE_STD		0x00	/**< Standard inquiry mode (without RSSI) */
#define BT_HCI_INQUIRY_MODE_RSSI	0x01	/**< Extended inquiry mode (with RSSI) */
/*******************************************************************************
 * bt_hci_read_inquiry_mode
 ******************************************************************************/
/**
 * @brief This cmd reads the inquiry mode (with or wihtout RSSI).
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 *
 * @return #BT_HCI_INQUIRY_MODE_STD or #BT_HCI_INQUIRY_MODE_RSSI or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_read_inquiry_mode(struct btstack *stack, struct bt_hci_cmd_response *response);


/*******************************************************************************
 * bt_hci_write_inquiry_mode
 ******************************************************************************/
/**
 * @brief This cmd writes the inquiry mode (with or wihtout RSSI).
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param mode #BT_HCI_INQUIRY_MODE_STD or #BT_HCI_INQUIRY_MODE_RSSI
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_write_inquiry_mode(struct btstack *stack, struct bt_hci_cmd_response *response, u_char mode);

#define BT_HCI_INQUIRY_SCAN_STD     0x00    /**< Standard scan */
#define BT_HCI_INQUIRY_SCAN_INTERLACED   0x01    /**< Interlaced scan */
/*******************************************************************************
 * bt_hci_read_inquiry_scan_type
 ******************************************************************************/
/**
 * @brief This cmd reads the inquiry scan type.
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 *
 * @return #BT_HCI_INQUIRY_SCAN_STD or #BT_HCI_INQUIRY_SCAN_INTERLACED or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_read_inquiry_scan_type(struct btstack *stack, struct bt_hci_cmd_response *response);


/*******************************************************************************
 * bt_hci_write_inquiry_scan_type
 ******************************************************************************/
/**
 * @brief This cmd writes the inquiry scan type.
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param mode #BT_HCI_INQUIRY_SCAN_STD or #BT_HCI_INQUIRY_SCAN_INTERLACED
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_write_inquiry_scan_type(struct btstack *stack, struct bt_hci_cmd_response *response, u_char mode);

//@}


/*******************************************************************************
 ###############################################################################
 * InformationalParameters HCI Commands
 ###############################################################################
 ******************************************************************************/
/*! \name InformationalParameters HCI Commands
 * Tries to send a #HCI_COMMAND_DATA_PACKET, the OpcodeGroup is #HCI_OGF_INFORMATIONAL_PARAMS
 */


//@{


/*******************************************************************************
 * bt_hci_read_local_version_information
 ******************************************************************************/
/**
 * @brief This command will read the values for the version information for the 
 * local Bluetooth device. The HCI Version information defines the version 
 * information of the HCI layer. The LMP Version information defines the version
 * of the LMP. The Manufacturer_Name information indicates the manufacturer of 
 * the local device. 
 * 
 * The HCI Revision and LMP Subversion are implementation dependant.
 * 
 * If the stack does not respond in the given time, the connections is rejected.
 * The default value is 0x1F40 (=5.00 s)
 *
 * @param stack The bt-stack to use
 * @param response A bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param version A bt_hci_local_information_result struct that holds the return 
 * parameters
 * 
 * version->hciversion
 * version->hcirevision
 * version->lmpversion
 * version->manufacturername
 * version->lmpsubversion
 * 
 * @return Waittime = time * 0.625msec (1 Baseband slot) range: 0x0001 - 0xB540
 *  if everything is fine and the call was sync or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
 long bt_hci_read_local_version_information(struct btstack *stack, struct bt_hci_cmd_response *response, struct bt_hci_local_version_result *version);


/*******************************************************************************
 * bt_hci_read_local_supported_commands
 ******************************************************************************/
/**
 * @brief This command reads the list of HCI commands supported for the local 
 * device (if suported).
 *  
 * If the stack does not respond in the given time, the connections is rejected.
 * The default value is 0x1F40 (=5.00 s)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 *
 * @return Waittime = time * 0.625msec (1 Baseband slot) range: 0x0001 - 0xB540
 *  if everything is fine and the call was sync or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
 long bt_hci_read_local_supported_commands(struct btstack *stack, struct bt_hci_cmd_response *response);


/*******************************************************************************
 * bt_hci_read_local_supported_features
 ******************************************************************************/
/**
 * @brief This command requests a list of the supported features for the local 
 * device. This command will return a list of the LMP features.
 *  
 * If the stack does not respond in the given time, the connections is rejected.
 * The default value is 0x1F40 (=5.00 s)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 *
 * @return Waittime = time * 0.625msec (1 Baseband slot) range: 0x0001 - 0xB540
 *  if everything is fine and the call was sync or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
 long bt_hci_read_local_supported_features(struct btstack *stack, struct bt_hci_cmd_response *response, u_char *features);


/*******************************************************************************
 * bt_hci_read_local_extended_features
 ******************************************************************************/
/**
 * @brief The HCI_Read_Local_Extended_Features command returns the requested 
 * page of the extended LMP features.
 *  
 * If the stack does not respond in the given time, the connections is rejected.
 * The default value is 0x1F40 (=5.00 s)
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 *
 * @return Waittime = time * 0.625msec (1 Baseband slot) range: 0x0001 - 0xB540
 *  if everything is fine and the call was sync or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
 long bt_hci_read_local_extended_features(struct btstack *stack, struct bt_hci_cmd_response *response);


 /*******************************************************************************
 * bt_hci_read_buffer_size
 ******************************************************************************/
/**
 * @brief This cmd reads the max. number of {acl|sco}-data-pkts, the controller can handle
 * and the max. size of an {acl|sco}-data-pkt the controller will ever send!
 *
 * \attention This command may NOT be called from an application !\n
 * It is internal for the stack and is only used after a reset to get these
 * informational parameters and must be called before any data is sent!
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 *
 * @return 0 if everything is fine, or possible errors: TODO: where are the return values??
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_read_buffer_size(struct btstack *stack, struct bt_hci_cmd_response *response);


/*******************************************************************************
 * bt_hci_read_bt_addr
 ******************************************************************************/
/**
 * @brief This cmd reads the local bluetooth address
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param addr a pointer to the struct where the address will be stored
 *
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_read_bt_addr(struct btstack *stack, struct bt_hci_cmd_response *response, bt_addr_t addr);
//@}


/*******************************************************************************
 ###############################################################################
 * StatusParameters HCI Commands
 ###############################################################################
 ******************************************************************************/
/*! \name StatusParameters HCI Commands
 * Tries to send a #HCI_COMMAND_DATA_PACKET, the OpcodeGroup is #HCI_OGF_HOST_CONTROL
 */


//@{


/*******************************************************************************
 * bt_hci_get_link_quality
 ******************************************************************************/
/**
 * @brief This command gets the link quality of an active connection.

 * Note: The link quality is between 0x00 and 0xff, higher values are better quality
 *
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle The (internal) connection handle
 * @param link_quality Pointer where the link_quality will be filled in!
 * 
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_get_link_quality(struct btstack *stack,  struct bt_hci_cmd_response *response, bt_hci_con_handle_t con_handle, u_char *link_quality);
//@}

/*******************************************************************************
 * bt_hci_read_rssi
 ******************************************************************************/
/**
 * @brief This command reads the rssi value of an active connection.

 * Note: The rssi value-range: -127 to +20, Units: dBm. Since the char type
 * is not necessarily defined as signed we pass a pointer of type short (signed)!
 *
 * The rssi-value will be stored in the rssi-Pointer
 * This is done because the rssi value is signed and this does not complies
 * with the default return police
 * 
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle The (internal) connection handle
 * @param rssi Pointer where the rssi-value will be filled in!
 * 
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_read_rssi(struct btstack *stack,  struct bt_hci_cmd_response *response, bt_hci_con_handle_t con_handle, short *rssi);
//@}


/*******************************************************************************
 * bt_hci_read_clock
 ******************************************************************************/
/**
 * @brief This command will read the estimate of the value of the Bluetooth Clock
 *        The BT Clock ticks every 0.3125 ms = 3.2 kHz
 *
 * @note The BTnode Rev 3 Zeevo returns the BT Clock divided by four. The returned
 *       time therefore has a precision of 1.25 ms. To somehow match the BT
 *       specifiation, the returned number is multiplied by four in the stack
 *       hence resulting in the two lowest bits always being zero.
 *
 * @note The BTnode Rev 3 Zeevo does not support retrieval of a remote clock
 *       but this can be accomplished by using bt_hci_read_clock( local ),
 *       bt_hci_read_clock_offset( remote device ) and receiving one packet with
 *       the complete BT clock from the remote device. The BT Clock offset is 
 *       within 0-2^14 * 1.25 ms ~ 30 seconds
 *       
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param which_clock The clock to use. 0 = Internal, 1 = Remote
 * @param con_handle The (internal) connection handle if remote clock is read
 * @param accuracy a u_short to store the retrieved accuracy if not NULL
 * 
 * @return BT Clock (28 bit) or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_read_clock(struct btstack *stack, struct bt_hci_cmd_response *response,
    u_char which_clock, bt_hci_con_handle_t app_con_handle, u_short *accuracy);

/*******************************************************************************
 ###############################################################################
 * VendorSpecific HCI Commands
 ###############################################################################
 ******************************************************************************/
/*! \name VendorSpecific HCI Commands
 * 
 */


//@{

/**
 * Defines baud rate of 19200
 */
#define BT_HCI_B19200 5

/**
 * Defines baud rate of 38400
 */
#define BT_HCI_B38400 4

/**
 * Defines baud rate of 57600
 */
#define BT_HCI_B57600 3

/**
 * Defines baud rate of 115200
 */
#define BT_HCI_B115200 2

/**
 * Defines baud rate of 230400
 */
#define BT_HCI_B230400 1

/**
 * Defines baud rate of 460800
 */
#define BT_HCI_B460800 0
/*******************************************************************************
 * bt_hci_set_baudrate
 ******************************************************************************/
/**
 * Sets the desired baudrate.
 * This command is specific for the uart layer!
 *
 * Possible Baudrate:
 * - #BT_HCI_B57600
 * - #BT_HCI_B115200
 * - #BT_HCI_B230400
 * - #BT_HCI_B460800
 *
 * @param stack The bt-stack to use
 * @param baudrate A possible Baudrate
 *
 * @return 0 if everything was ok, or an errorcode on error
 */
long bt_hci_set_baudrate(struct btstack *stack, u_char baudrate);
//@}

/*******************************************************************************
* bt_hci_read_abs_rssi
******************************************************************************/
/**
* @brief This command reads the absolute rssi value of an active connection.
 * 
 * Note: The absolute rssi value-range: -127 to +20, Units: dBm. 
 *
 * The absolute rssi-value will be stored in the abs_rssi-Pointer
 * 
 * @param stack The bt-stack to use
 * @param response a bt_hci_cmd_response struct for async. calls or #BT_HCI_SYNC
 * @param con_handle The (internal) connection handle
 * @param abs_rssi Pointer where the absolute rssi-value will be filled in!
 * 
 * @return 0 or possible errors:
 *  - \ref Bt_Error_Codes
 *  - #BT_ERR_BT_MODULE
 *  - #BT_ERR_UART_COM
 *  - #BT_ERR_USB_COM
 */
long bt_hci_read_abs_rssi(struct btstack *stack,  struct bt_hci_cmd_response *response, bt_hci_con_handle_t con_handle, char *abs_rssi);
//@}
