/*
 * Copyright (C) 2000-2005 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */

#ifndef _XBANKDATA_H_
#define _XBANKDATA_H_

/**
 * \file xbankdata.h
 *
 * \brief This file contains functionality to store and maintain programs
 * in banked memory.
 */

#include <sys/types.h>
#include <hardware/ram_banks.h>
#include <support/bootloader.h>
#include <mhop/rpc.h>

#define XBANK_PROG_START        XBANK_PRSTART
#define XBANK_PROG_HEADER_START XBANK_PROG_START
#define XBANK_PROG_HEADER_LEN   (MEM_LEN_HEADER)
#define XBANK_PROG_DATA_START   (XBANK_PROG_HEADER_START + XBANK_PROG_HEADER_LEN + CRC_LENGTH)
#define XBANK_PROG_END          ((129L * 1024)-1)

#define XBANK_RECORD_HEADER_LEN 6

#define XBANK_PROG_TYPE_TARGET  REPROG_TARGET_PROGRAM_DATA
#define XBANK_PROG_TYPE_DSN     REPROG_BTNODE_PROGRAM_DATA

#define XBANK_CMD_GET_PROGINFO 1
#define XBANK_CMD_GET_STATUS   2
#define XBANK_CMD_SET_PROGTYPE 3
#define XBANK_CMD_SET_PROGNAME 4
#define XBANK_CMD_SET_PROGVER  5
#define XBANK_INVALIDATE_PROG  6

#define XBANK_MAX_PROGNAME_LEN  30

/// Program info length.
#define XBANK_PROG_INFO_LEN     sizeof(xbank_prog_info_t)
/// Program information struct.
typedef struct _xbank_prog_info_s {
	u_char type;
	u_long size;
	u_long boot_addr;
	u_long version;
	char name[XBANK_MAX_PROGNAME_LEN+1];
} xbank_prog_info_t;

/**
 * \brief Prototyp of the enable flash callback.
 *
 * The enable flash callback is called after the whole image was stored.
 * If the callback returns 0, the flash flag will not be enabled,
 * else it will be enabled. If the flash flag is enabled, the image 
 * will be flashed from the bootloader after resetting.
 */
#define XBANK_ENABLE_FLASH_CB u_char (*enable_flash_cb) (xbank_prog_info_t * info)

typedef struct _xbank_stack_s {
    u_char program_lock;
    u_char program_valid;
    xbank_prog_info_t program_info;
    u_long program_fuses;
    u_long program_pos;       // current read/write address
    u_long program_recpos;    // address of current progdata record
    u_long program_recaddr;
    u_long program_reclen;
    u_char proc_nr;
    XBANK_ENABLE_FLASH_CB;
} xbank_stack_t;

xbank_stack_t* _xbank;

// TODO: doxygen
void xbank_get_prog_info(xbank_prog_info_t *prog_info);
void xbank_set_prog_info(xbank_prog_info_t *prog_info);
void xbank_set_prog_boot_addr(u_long boot_addr);

/**
 * \brief Prepares xbank to store a new program. 
 */
void xbank_init_prog_write(void);

/**
 * \brief Starts a new program record.
 * 
 * \param addr The start address of the data the record will hold.
 * \return 0 on success, 1 if program size exceeds maximum
 */
u_char xbank_write_prog_record(u_long addr);

/**
 * \brief Write data into current program record.
 *
 * \param buffer Buffer from which data is read.
 * \param len Length of buffer
 * \return 0 on success, 1 if program size exceeds maximum
 */
u_char xbank_write_prog_data(u_char *buffer, u_short len);

/**
 * \brief Write data crc to complete program in memory.
 *
 * \param buffer Buffer that holds the crc or NULL if a crc should 
 * be generated based on current program records.
 * \return 0 on success, 1 if program size exceeds maximum
 */
u_char xbank_write_prog_crc(u_char *buffer);


/**
 * \brief Prepares xbank to read a new program. 
 */
void xbank_init_prog_read(void);

/**
 * \brief Read next program record.
 *
 * \param addr Record data. Start address of the data in the record.
 * \param len Record data. Number of bytes the record contains.
 * \return 0 on success, 1 if all records have been read
 */
u_char xbank_read_prog_record(u_long *addr, u_long *len);

/**
 * \brief Read next len bytes from the current record.
 *
 * \param buffer Buffer for the data.
 * \param len Number of bytes to copy.
 */
void xbank_read_prog_data(u_char *buffer, u_short len);

/**
 * \brief Validates program.
 *
 * Checks memory for a valid program and caches the result.
 * The program check involves rather expensive crc computations.
 * However, the result is cached and you have to call it only once 
 * when you store a new program. Use #xbank_check_program() 
 * afterwards for fast query.
 * 
 * \return 0 if there was a valid program, 1 otherwise.
 */
u_char xbank_validate_program(void);

/**
 * \brief Checks wether there is a valid program in memory. 
 *
 * \return 0 on success, 1 otherwise.
 */
u_char xbank_check_program(void);

/**
 * \brief Deletes the current program.
 */
void xbank_invalidate_program(void);

/**
 * \brief Checks and sets progam access lock. 
 *
 * \return 0 on success, 1 otherwise.
 */
u_char xbank_get_program_lock(void);

/**
 * \brief Frees the progam access lock.
 */
void xbank_free_program_lock(void);

inline void xbank_update_prog_header(void);

inline void xbank_set_prog_name(char *name);

/**
 * \brief Registers the enable flash callback.
 *
 * If this function is not used, the flash flag will be always
 * enabled after the whole image was stored.
 *
 * \see XBANK_ENABLE_FLASH_CB
 */
void xbank_register_enable_flash_callback(XBANK_ENABLE_FLASH_CB);

void xbank_init(void);

#endif //_XBANKDATA_H_
